#!/usr/bin/env python3
# license-solver
# Copyright(C) 2021 Red Hat, Inc.
#
# This program is free software: you can redistribute it and / or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

"""Main class which work with detecting and creating output."""

import os
import sys
import json
import logging
import requests

from typing import List, Tuple, Dict, Any, Optional, Union
from os import DirEntry

from .classifiers import Classifiers
from .licenses import Licenses
from .package import Package, _detect_version_and_delete
from .json_solver import JsonSolver
from .comparator import _delete_brackets, _delete_brackets_and_content
from .output_creator import OutputCreator
from .exceptions import UnableOpenFileData

_LOGGER = logging.getLogger(__name__)


class Solver:
    """Class pass all detected files and try to detect all necessary data."""

    def __init__(self, github: bool = False) -> None:
        """Init class variables and open JSON file of license aliases."""
        self.license_dictionary: Dict[str, Any] = dict()
        self.classifiers: Classifiers = Classifiers()
        self.licenses: Licenses = Licenses()
        self.output: OutputCreator = OutputCreator(github)

        file_path = os.path.join(os.path.dirname(os.path.realpath(__file__)), "data", "license_dictionary.json")
        try:
            with open(file_path) as f:
                self.license_dictionary = json.load(f).get("data")
                _LOGGER.debug("File license_dictionary.json was successful loaded")
        except Exception:
            raise UnableOpenFileData

    def solve_from_file(self, input_file: Union[Dict[str, Any], str]) -> None:
        """
        Solver from file.

        :param input_file: file path
        :return: None
        """
        if isinstance(input_file, str):
            _LOGGER.debug("Parsing file: %s", input_file)
            if not self._check_if_json(input_file):
                _LOGGER.warning("Input file is not valid. SKIPPED")
                return
            # path to file
            try:
                with open(input_file) as f:
                    json_solver = JsonSolver(json.load(f), f.name)  # type: ignore[call-arg]
                    json_solver.get_info_attribute()
                    _LOGGER.debug("Loaded file %s", input_file)
            except Exception as e:
                _LOGGER.error("Broken or can't find file: %s\nerror: %s.", input_file, e)

        elif isinstance(input_file, dict):
            _LOGGER.debug("Parsing dictionary.")
            # dictionary parsing
            json_solver = JsonSolver(input_file, "dictionary_input")  # type: ignore[call-arg]

        else:
            _LOGGER.warning("Not supported type: %s. SKIPPED", type(input_file))
            return

        try:
            if json_solver is None:
                return
        except Exception:
            return

        package = Package()

        self._get_classifier_and_license(json_solver, package)
        self.output.add_package(package)

    def solve_from_directory(self, input_directory: str) -> None:
        """
        Solve from directory.

        :param input_directory: directory path
        :return: None
        """
        _LOGGER.debug("Start parsing directory %s.", input_directory)
        file_path: DirEntry  # type: ignore[type-arg]
        for file_path in os.scandir(input_directory):
            if file_path.is_file():
                self.solve_from_file(file_path.path)
            else:
                _LOGGER.debug("Subdirectory SKIPPED %s.", file_path)

    def solve_from_pypi(self, package_name: str, package_version: Optional[str]) -> None:
        """
        Solve from PyPI.

        :param package_name: package name to solver
        :param package_version: package version to solver
        :return: None
        """
        if package_version:
            url = f"https://pypi.org/pypi/{package_name}/{package_version}/json"
            response = requests.get(url, headers={"User-Agent": "license-solver"})

            if response.status_code != 200:
                _LOGGER.warning("Package %r with version %r was not found on PyPI.", package_name, package_version)
                print(f"Package {package_name} with {package_version} was not found on PyPI.", file=sys.stderr)
                return
        else:
            # get latest licenses
            url = f"https://pypi.org/pypi/{package_name}/json"
            response = requests.get(url, headers={"User-Agent": "license-solver"})

            if response.status_code != 200:
                _LOGGER.warning("Package %r was not found on PyPI.", package_name)
                print(f"Package {package_name} was not found on PyPI.", file=sys.stderr)
                return

        # convert to dictionary
        res = json.loads(response.text)
        # solver like file
        self.solve_from_file(res)
        self.solve_from_file(res.get("info"))

    def _get_classifier_and_license(self, json_file: JsonSolver, package: Package) -> None:
        """
        Get classifier and license groups and save them to parameter package.

        :param json_file: json class which hold data from file
        :param package: class package which will hold all package data
        :return: None
        """
        package.set_file_path(json_file.path)

        package.set_package_name(json_file.get_package_name())
        package.set_version(json_file.get_package_version())

        license_name = json_file.get_license_name()
        package.set_license(self._get_license_group(license_name))

        classifier_name = json_file.get_classifier_name()
        package.set_classifier(self._get_classifier_group(classifier_name))

    def _get_license_group(self, license_name: Optional[str]) -> Tuple[List[str], bool]:
        """
        Search for a group of entered license name.

        :param license_name: name of license to find in class license_list
        :return: Tuple[List[str], bool]:
        """
        # undetected license
        if license_name is None:
            return list(["UNDETECTED"]), False

        # UNKNOWN license name
        if license_name.lower() == "unknown":
            return list(["UNKNOWN"]), False

        # pass license list
        for lic_li in self.licenses.licenses_list:
            lic_li_lower = [x.lower() for x in lic_li]
            if (
                license_name.lower() in lic_li_lower
                or _delete_brackets(license_name).lower() in lic_li_lower
                or _delete_brackets_and_content(license_name).lower() in lic_li_lower
            ):
                return lic_li, True

        # try to found license without version or license in dictionary
        for lic_li in self.licenses.licenses_list:
            license_name_no_version, _ = _detect_version_and_delete(lic_li[len(lic_li) - 1])

            if self.license_dictionary.get(license_name.lower()) in lic_li:
                # license found in license dictionary
                _license = self.license_dictionary.get(license_name.lower())
                _license_li = [x for x in self.licenses.licenses_list if _license in x][0]
                return _license_li, True
            elif license_name_no_version == license_name:
                return list([license_name]), True

        return list(["UNDETECTED"]), False

    def _get_classifier_group(self, classifier_name: Optional[List[str]]) -> Optional[List[str]]:
        """
        Search for a group of entered classifier name.

        :param classifier_name: name of license to find in class classifier list
        :return: None
        """
        if classifier_name is None:
            return None

        cla_li: List[str]
        for cla_li in self.classifiers.classifiers_list:
            # lowercase lists and compare
            cla_li_lower = [x.lower() for x in cla_li.copy()]
            classifier_name_lower = [x.lower() for x in classifier_name.copy()]

            if list(set(cla_li_lower) & set(classifier_name_lower)):
                return cla_li

        return None

    def print_output(self, indent: int = -1) -> None:
        """Print final output on STDOUT."""
        self.output.print(indent)

    def save_output(self, file_name: str, indent: int = -1) -> None:
        """Save solver result to JSON."""
        f = open(file_name, "w")
        if indent < 0:
            f.write(json.dumps(self.output.file))
        else:
            f.write(json.dumps(self.output.file, indent=indent))
        f.close()

    def get_output_dict(self, **kw: Any) -> Dict[str, Any]:
        """Return dictionary from OutputCreator class."""
        condition = True if kw["package_name"] and kw["package_version"] else False
        return (  # type: ignore[no-any-return]
            self.output.file[kw["package_name"]][kw["package_version"]] if condition else self.output.file
        )

    @staticmethod
    def get_empty_dict() -> Dict[str, Any]:
        """Return empty dictionary for license."""
        return {
            "license": {
                "full_name": "UNDETECTED",
                "identifier": "UNDETECTED",
                "identifier_spdx": "UNDETECTED",
            },
            "license_version": "UNDETECTED",
            "warning": True,
        }

    @staticmethod
    def _check_if_json(input_file: str) -> bool:
        """Check if input file is JSON type."""
        if not input_file.endswith(".json"):
            _LOGGER.warning("File %s is not JSON type. SKIPPED", input_file)
            return False
        return True



#!/usr/bin/env python3
# license-solver
# Copyright(C) 2021 Red Hat, Inc.
#
# This program is free software: you can redistribute it and / or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
# type: ignore[misc]

"""solver-license-jon CLI."""
import os
import sys
import click
import logging

from thoth.common import init_logging
from thoth.license_solver.solver import Solver
from thoth.license_solver import __version__ as license_solver_version

init_logging()
_LOGGER = logging.getLogger("thoth.license_solver")


def _print_version(ctx: click.Context, _, value: str) -> None:
    """Print license-solver version and exit."""
    if not value or ctx.resilient_parsing:
        return

    click.echo(license_solver_version)
    ctx.exit()


class OptionEatAll(click.Option):
    """
    Eat all argument of parameter.

    source: https://stackoverflow.com/a/48394004
    """

    def __init__(self, *args, **kwargs):
        """Initialize."""
        self.save_other_options = kwargs.pop("save_other_options", True)
        nargs = kwargs.pop("nargs", -1)
        assert nargs == -1, "nargs, if set, must be -1 not {}".format(nargs)
        super(OptionEatAll, self).__init__(*args, **kwargs)
        self._previous_parser_process = None
        self._eat_all_parser = None

    def add_to_parser(self, parser, ctx):
        """Add to parser."""

        def parser_process(value, state):
            """Parser process."""
            # method to hook to the parser.process
            done = False
            value = [value]
            if self.save_other_options:
                # grab everything up to the next option
                while state.rargs and not done:
                    for prefix in self._eat_all_parser.prefixes:
                        if state.rargs[0].startswith(prefix):
                            done = True
                    if not done:
                        value.append(state.rargs.pop(0))
            else:
                # grab everything remaining
                value += state.rargs
                state.rargs[:] = []
            value = tuple(value)

            # call the actual process
            self._previous_parser_process(value, state)

        retval = super(OptionEatAll, self).add_to_parser(parser, ctx)
        for name in self.opts:
            our_parser = parser._long_opt.get(name) or parser._short_opt.get(name)
            if our_parser:
                self._eat_all_parser = our_parser
                self._previous_parser_process = our_parser.process
                our_parser.process = parser_process
                break
        return retval


@click.command(context_settings=dict(max_content_width=160))
@click.pass_context
@click.option(
    "-v",
    "--verbose",
    is_flag=True,
    help="Be verbose about what's going on.",
    envvar="THOTH_SOLVER_LICENSE_JOB_DEBUG",
)
@click.option(
    "--version",
    is_flag=True,
    is_eager=True,
    callback=_print_version,
    expose_value=False,
    help="Print license-solver version and exit.",
)
@click.option(
    "-f",
    "--file",
    type=tuple,
    cls=OptionEatAll,
    help="Get license from file.",
    envvar="THOTH_SOLVER_LICENSE_JOB_FILE",
)
@click.option(
    "-d",
    "--directory",
    type=tuple,
    cls=OptionEatAll,
    help="Get licenses from folder.",
    envvar="THOTH_SOLVER_LICENSE_JOB_DIRECTORY",
)
@click.option(
    "-pn",
    "--package-name",
    type=tuple,
    cls=OptionEatAll,
    help='Get license from latest PyPI release or use with "--package-version" for specific version.',
    envvar="THOTH_SOLVER_LICENSE_PACKAGE_NAME",
)
@click.option(
    "-pv",
    "--package-version",
    nargs=1,
    type=str,
    help="Get license with specific version.",
    envvar="THOTH_SOLVER_LICENSE_PACKAGE_VERSION",
)
@click.option(
    "-o",
    "--output",
    type=str,
    help="Save output to JSON file.",
    envvar="THOTH_SOLVER_LICENSE_OUTPUT",
)
@click.option(
    "-np",
    "--no-print",
    is_flag=True,
    help="No print on STDOUT.",
    envvar="THOTH_SOLVER_LICENSE_NO_PRINT",
)
@click.option(
    "-pp",
    "--pretty-printing",
    type=int,
    nargs=1,
    default=-1,
    show_default=True,
    help="Save/print result with prettier look.",
    envvar="THOTH_SOLVER_LICENSE_INDENT",
)
@click.option(
    "-gch",
    "--github-check",
    is_flag=True,
    help="Check licenses with Github repository.",
    envvar="THOTH_SOLVER_LICENSE_GITHUB_CHECK",
)
def cli(
    ctx: click.Context,
    directory: tuple,
    file: tuple,
    package_name: str,
    package_version: str,
    output: str,
    no_print: bool,
    pretty_printing: int,
    github_check: bool = False,
    verbose: bool = False,
) -> None:
    """
    License solver.

    License-solver handles license detection and classifier detection from metadata provided by PyPI.
    The program prints the result in the form of JSON on STDOUT.
    """
    if verbose:
        _LOGGER.setLevel(logging.DEBUG)
        _LOGGER.debug("Debug mode is on")

    license_solver = Solver(github_check)

    # package argument
    if package_name is not None:
        if len(package_name) > 1:
            if package_version:
                _LOGGER.warning("Can't insert version to multiple package_name entry.")
                exit(1)

            for pn in package_name:
                license_solver.solve_from_pypi(pn, package_version)

        elif len(package_name) == 1:
            license_solver.solve_from_pypi(package_name[0], package_version)
        elif package_version:
            print(ctx.get_help(), "\n\n--package-version is used with --package-name.", file=sys.stderr)
            exit(1)

    # directory argument
    if directory:
        for d in directory:
            if not os.path.isdir(d):
                _LOGGER.warning("Not a valid directory %r [SKIPPED].", d)
                continue

            _LOGGER.debug("Parsing directory: %s", d)
            license_solver.solve_from_directory(d)

    # file argument
    if file:
        for f in file:
            if not os.path.isfile(f):
                _LOGGER.warning("Not a valid file %r [SKIPPED].", f)
                continue

            _LOGGER.debug("Parsing file: %s", f)
            license_solver.solve_from_file(f)

    if output:
        license_solver.save_output(output, pretty_printing)

    if not no_print:
        license_solver.print_output(pretty_printing)


__name__ == "__main__" and cli()



#!/usr/bin/env python3
# license-solver
# Copyright(C) 2021 Red Hat, Inc.
#
# This program is free software: you can redistribute it and / or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

"""A class for work with loaded json files."""

import attr
from attr import attrib
import logging
from typing import Dict, Any, Union, Optional, List

_LOGGER = logging.getLogger("thoth.license_solver.json_solver")


@attr.s(slots=True)
class JsonSolver:
    """Class JsonSolver extend class LicenseSolver, help inquiry json data from metadata."""

    json_file = attrib(type=Dict[str, Any])
    path = attrib(type=str)

    def get_info_attribute(self) -> None:
        """Get info from metadata."""
        res = self.json_file.get("info")
        if res is not None:
            self.json_file = res  # type: ignore[assignment]

    def get_package_name(self) -> Optional[Any]:
        """Get package name from metadata."""
        return self.json_file.get("Name") or self.json_file.get("name")

    def get_package_version(self) -> Optional[Any]:
        """Get package version from metadata."""
        return self.json_file.get("Version") or self.json_file.get("version")

    def get_license_name(self) -> Optional[Any]:
        """Get license name from metadata."""
        return self.json_file.get("License") or self.json_file.get("license")

    def get_classifier_name(self) -> Union[List[Any], Any, None]:
        """Get classifier name from metadata."""
        return self.json_file.get("Classifier") or self.json_file.get("classifiers")



#!/usr/bin/env python3
# license-solver
# Copyright(C) 2021 Red Hat, Inc.
#
# This program is free software: you can redistribute it and / or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

"""Tests related to class OutputCreator."""

from typing import List, Dict
from thoth.license_solver.output_creator import OutputCreator
from thoth.license_solver.package import Package


class TestOutputCreator:
    """Test OutputCreator."""

    output_creator: OutputCreator = OutputCreator(False)

    @staticmethod
    def create_package(
        name: str,
        version: str,
        license_name: Dict[str, str],
        license_version: str,
        license_bool: bool,
        classifier: List[str],
    ) -> Package:
        """Create package."""
        license_list: List[str, str] = []

        if license_name:
            license_list = [license_name["full_name"], license_name["identifier_spdx"], license_name["identifier"]]

        package = Package()
        package.set_package_name(name)
        package.set_version(version)
        package.set_license((license_list, license_bool))
        package.set_license_version(license_version)
        package.set_classifier(classifier)

        return package

    def test_add_package_solo(self) -> None:
        """Test add_package solo versions."""
        package_1 = self.create_package(
            "test_1",
            "1.0",
            {"full_name": "MIT License", "identifier_spdx": "MIT", "identifier": "MIT"},
            "LICENSE-WITHOUT-VERSION",
            False,
            ["License :: OSI Approved :: MIT License", "MIT License"],
        )
        self.output_creator.add_package(package_1)

        assert self.output_creator.file == {
            "test_1": {
                "1.0": {
                    "license": {"full_name": "MIT License", "identifier_spdx": "MIT", "identifier": "MIT"},
                    "license_version": "LICENSE-WITHOUT-VERSION",
                    "classifier": [["License :: OSI Approved :: MIT License", "MIT License"]],
                    "warning": False,
                }
            }
        }

    def test_add_package_version_solo(self) -> None:
        """Test add_package solo versions."""
        package = self.create_package(
            "test_1",
            "1.1",
            {"full_name": "MIT License", "identifier_spdx": "MIT", "identifier": "MIT"},
            "LICENSE-WITHOUT-VERSION",
            False,
            ["License :: OSI Approved :: MIT License", "MIT License"],
        )
        self.output_creator.add_package(package)

        assert self.output_creator.file == {
            "test_1": {
                "1.0": {
                    "license": {"full_name": "MIT License", "identifier_spdx": "MIT", "identifier": "MIT"},
                    "license_version": "LICENSE-WITHOUT-VERSION",
                    "classifier": [["License :: OSI Approved :: MIT License", "MIT License"]],
                    "warning": False,
                },
                "1.1": {
                    "license": {"full_name": "MIT License", "identifier_spdx": "MIT", "identifier": "MIT"},
                    "license_version": "LICENSE-WITHOUT-VERSION",
                    "classifier": [["License :: OSI Approved :: MIT License", "MIT License"]],
                    "warning": False,
                },
            }
        }

    def test_add_package(self) -> None:
        """Test add_package."""
        package = self.create_package(
            "test_2",
            "1.0",
            {"full_name": "MIT License", "identifier_spdx": "MIT", "identifier": "MIT"},
            "LICENSE-WITHOUT-VERSION",
            False,
            ["License :: OSI Approved :: MIT License", "MIT License"],
        )
        self.output_creator.add_package(package)

        assert self.output_creator.file == {
            "test_1": {
                "1.0": {
                    "license": {"full_name": "MIT License", "identifier_spdx": "MIT", "identifier": "MIT"},
                    "license_version": "LICENSE-WITHOUT-VERSION",
                    "classifier": [["License :: OSI Approved :: MIT License", "MIT License"]],
                    "warning": False,
                },
                "1.1": {
                    "license": {"full_name": "MIT License", "identifier_spdx": "MIT", "identifier": "MIT"},
                    "license_version": "LICENSE-WITHOUT-VERSION",
                    "classifier": [["License :: OSI Approved :: MIT License", "MIT License"]],
                    "warning": False,
                },
            },
            "test_2": {
                "1.0": {
                    "license": {"full_name": "MIT License", "identifier_spdx": "MIT", "identifier": "MIT"},
                    "license_version": "LICENSE-WITHOUT-VERSION",
                    "classifier": [["License :: OSI Approved :: MIT License", "MIT License"]],
                    "warning": False,
                }
            },
        }

    def test_add_package_version(self) -> None:
        """Test add_package with version."""
        package = self.create_package(
            "test_2",
            "1.1",
            {"full_name": "MIT License", "identifier_spdx": "MIT", "identifier": "MIT"},
            "LICENSE-WITHOUT-VERSION",
            False,
            ["License :: OSI Approved :: Apache Software License", "Apache Software License"],
        )
        self.output_creator.add_package(package)

        assert self.output_creator.file == {
            "test_1": {
                "1.0": {
                    "license": {"full_name": "MIT License", "identifier_spdx": "MIT", "identifier": "MIT"},
                    "license_version": "LICENSE-WITHOUT-VERSION",
                    "classifier": [["License :: OSI Approved :: MIT License", "MIT License"]],
                    "warning": False,
                },
                "1.1": {
                    "license": {"full_name": "MIT License", "identifier_spdx": "MIT", "identifier": "MIT"},
                    "license_version": "LICENSE-WITHOUT-VERSION",
                    "classifier": [["License :: OSI Approved :: MIT License", "MIT License"]],
                    "warning": False,
                },
            },
            "test_2": {
                "1.0": {
                    "license": {"full_name": "MIT License", "identifier_spdx": "MIT", "identifier": "MIT"},
                    "license_version": "LICENSE-WITHOUT-VERSION",
                    "classifier": [["License :: OSI Approved :: MIT License", "MIT License"]],
                    "warning": False,
                },
                "1.1": {
                    "license": {"full_name": "MIT License", "identifier_spdx": "MIT", "identifier": "MIT"},
                    "license_version": "LICENSE-WITHOUT-VERSION",
                    "classifier": [["License :: OSI Approved :: Apache Software License", "Apache Software License"]],
                    "warning": True,
                },
            },
        }

    def test_add_package_same_package_and_version(self) -> None:
        """Test add_package with same package and version."""
        package_same_package = self.create_package(
            "test_2",
            "1.1",
            {"full_name": "MIT License", "identifier_spdx": "MIT", "identifier": "MIT"},
            "LICENSE-WITHOUT-VERSION",
            False,
            ["License :: OSI Approved :: Apache Software License", "Apache Software License"],
        )
        package_same_version = self.create_package(
            "test_2",
            "1.1",
            {"full_name": "MIT License", "identifier_spdx": "MIT", "identifier": "MIT"},
            "LICENSE-WITHOUT-VERSION",
            False,
            ["License :: OSI Approved :: Apache Software License", "Apache Software License"],
        )
        self.output_creator.add_package(package_same_package)
        self.output_creator.add_package(package_same_version)

        assert self.output_creator.file == {
            "test_1": {
                "1.0": {
                    "license": {"full_name": "MIT License", "identifier_spdx": "MIT", "identifier": "MIT"},
                    "license_version": "LICENSE-WITHOUT-VERSION",
                    "classifier": [["License :: OSI Approved :: MIT License", "MIT License"]],
                    "warning": False,
                },
                "1.1": {
                    "license": {"full_name": "MIT License", "identifier_spdx": "MIT", "identifier": "MIT"},
                    "license_version": "LICENSE-WITHOUT-VERSION",
                    "classifier": [["License :: OSI Approved :: MIT License", "MIT License"]],
                    "warning": False,
                },
            },
            "test_2": {
                "1.0": {
                    "license": {"full_name": "MIT License", "identifier_spdx": "MIT", "identifier": "MIT"},
                    "license_version": "LICENSE-WITHOUT-VERSION",
                    "classifier": [["License :: OSI Approved :: MIT License", "MIT License"]],
                    "warning": False,
                },
                "1.1": {
                    "license": {"full_name": "MIT License", "identifier_spdx": "MIT", "identifier": "MIT"},
                    "license_version": "LICENSE-WITHOUT-VERSION",
                    "classifier": [["License :: OSI Approved :: Apache Software License", "Apache Software License"]],
                    "warning": True,
                },
            },
        }

    def test_check_duplicity(self) -> None:
        """Test check_duplicity."""
        package_old = {
            "license": ["Apache License 1.1", "Apache-1.1", "Apache 1.1"],
            "license_version": "1.1",
            "warning": False,
            "classifier": ["License :: OSI Approved :: Apache Software License", "Apache Software License"],
        }

        package_new = {
            "license": ["MIT License", "MIT"],
            "license_version": "LICENSE-WITHOUT-VERSION",
            "warning": False,
            "classifier": ["License :: OSI Approved :: MIT License", "MIT License"],
        }

        package_old_empty = {
            "license": None,
            "license_version": None,
            "warning": False,
            "classifier": ["License :: OSI Approved :: Apache Software License", "Apache Software License"],
        }

        self.output_creator._check_duplicity(package_old, package_old)
        assert package_old.get("warning") is False

        self.output_creator._check_duplicity(package_old, package_new)
        assert package_old.get("warning") is True

        package_old["warning"] = False  # reset default value
        self.output_creator._check_duplicity(package_old_empty, package_old)
        assert package_old_empty.get("warning") is False
        assert package_old_empty.get("license") == ["Apache License 1.1", "Apache-1.1", "Apache 1.1"]
        assert package_old_empty.get("license_version") == "1.1"



#!/usr/bin/env python3
# license-solver
# Copyright(C) 2021 Red Hat, Inc.
#
# This program is free software: you can redistribute it and / or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

"""Tests related to class JsonSolver."""

import json
import pytest
import os
from thoth.license_solver.json_solver import JsonSolver


class TestJsonSolver:
    """Test JsonSolver."""

    json_solver: JsonSolver
    json_solver_empty: JsonSolver

    @pytest.fixture(autouse=True)
    def _setup_json_solver_variables(self):
        """Set up class variables."""
        file_path = os.path.join(
            os.path.dirname(os.path.realpath(__file__)), "test_files", "json_solver", "load_json_file.json"
        )
        with open(file_path) as f:
            file_data = json.load(f)
        self.json_solver = JsonSolver(file_data, file_path)

        # create json_solver for empty file
        file_path_empty = os.path.join(os.path.dirname(os.path.realpath(__file__)), "test_files/json_solver/empty.json")
        with open(file_path_empty) as f:
            file_data_empty = json.load(f)

        self.json_solver_empty = JsonSolver(file_data_empty, file_path_empty)

    def test_get_package_name(self):
        """Test get_package_name."""
        assert self.json_solver.get_package_name() == "setuptools-scm"
        assert self.json_solver_empty.get_package_name() is None

    def test_get_package_version(self):
        """Test get_package_version."""
        assert self.json_solver.get_package_version() == "1.15.4"
        assert self.json_solver_empty.get_package_version() is None

    def test_get_license_name(self):
        """Test get_license_name."""
        assert self.json_solver.get_license_name() == "MIT"
        assert self.json_solver_empty.get_license_name() is None

    def test_get_classifier_name(self):
        """Test get_classifier_name."""
        classifier = [
            "Development Status :: 4 - Beta",
            "Intended Audience :: Developers",
            "License :: OSI Approved :: MIT License",
            "Programming Language :: Python",
            "Programming Language :: Python :: 3",
            "Programming Language :: Python :: 2.6",
            "Programming Language :: Python :: 2.7",
            "Programming Language :: Python :: 3.3",
            "Programming Language :: Python :: 3.4",
            "Programming Language :: Python :: 3.5",
            "Programming Language :: Python :: 3.6",
            "Topic :: Software Development :: Libraries",
            "Topic :: Software Development :: Version Control",
            "Topic :: System :: Software Distribution",
            "Topic :: Utilities",
        ]
        assert self.json_solver.get_classifier_name() == classifier
        assert self.json_solver_empty.get_classifier_name() is None



#!/usr/bin/env python3
# license-solver
# Copyright(C) 2021 Red Hat, Inc.
#
# This program is free software: you can redistribute it and / or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

"""Test related to class Comparator."""

from thoth.license_solver.comparator import Comparator
from thoth.license_solver.package import Package


class TestComparator:
    """Test Comparator."""

    comparator: Comparator = Comparator(False)

    # tests related to Compare().cmp()
    def test_cmp(self) -> None:
        """Test compare function in Comparator class."""
        package_1 = Package()
        assert self.comparator.cmp(package_1), "Empty package return true, because there is nothing to compare"
        package_1.set_license((["Apache License 1.1", "Apache-1.1", "Apache 1.1"], True))
        package_1.set_classifier(["License :: OSI Approved :: Apache Software License", "Apache Software License"])
        assert self.comparator.cmp(package_1), "Not matched apache license and classifier"

        package_2 = Package()
        package_2.set_license((["MIT License", "MIT", "MIT"], True))
        package_2.set_classifier(["License :: OSI Approved :: MIT License", "MIT License"])
        assert self.comparator.cmp(package_2), "Not matched MIT license and classifier"

        # test wrong license and classifier
        package_3 = Package()
        package_3.set_license((["MIT License", "MIT", "MIT"], True))
        package_3.set_classifier(["License :: OSI Approved :: Apache Software License", "Apache Software License"])
        assert self.comparator.cmp(package_3) is False, "License and Classifier can't match"

    def test_cmp_missing(self) -> None:
        """Test compare function in Comparator class and check result for missing license or classifier or both."""
        package_1 = Package()
        package_1.set_license((["Apache License 1.1", "Apache-1.1", "Apache 1.1"], True))
        assert self.comparator.cmp(package_1), "Must return true, because there is nothing to compare"
        # del package_1

        package_2 = Package()
        package_2.set_classifier(["License :: OSI Approved :: MIT License", "MIT License"])
        assert self.comparator.cmp(package_2), "Must return true, because there is nothing to compare"

    # tests related to Compare().search_in_dictionary()
    def test_search_in_dictionary(self) -> None:
        """Test function search_in_dictionary."""
        license_name = ['BSD 4-Clause "Original" or "Old" License']
        license_name_not_valid = ["Not Valid", "NoV"]
        classifier = ["License :: OSI Approved :: BSD License", "BSD License"]
        classifier_not_valid = ["Not valid", "Not Valid"]

        assert self.comparator.search_in_dictionary(license_name, classifier), "License was not found in dictionary"
        assert (
            self.comparator.search_in_dictionary(license_name_not_valid, classifier) is False
        ), "Can't match classfier with license"
        assert (
            self.comparator.search_in_dictionary(license_name, classifier_not_valid) is False
        ), "Classifier cant be find because is not defined in dictionary file"

    def test_search_in_dictionary_missing(self) -> None:
        """Test missing data/input in search_in_dictionary in Comparator."""
        assert self.comparator.search_in_dictionary(list(), list()) is False, "Nothing to found in license dictionary"



#!/usr/bin/env python3
# license-solver
# Copyright(C) 2021 Red Hat, Inc.
#
# This program is free software: you can redistribute it and / or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

"""Class create output dictionary."""

import json
import sys
import logging
from .comparator import Comparator
from .package import Package
from typing import Dict, Any

_LOGGER = logging.getLogger(__name__)


class OutputCreator:
    """Propose of this class is to create dictionary for all packages (input)."""

    def __init__(self, github: bool = False) -> None:
        """
        Init variables for OutputCreator.

        :param github:
        """
        self.file: Dict[Any, Any] = dict()
        self.comparator: Comparator = Comparator(github)

    def add_package(self, package: Package) -> None:
        """
        Add package to dictionary.

        :param package: Package data
        :return: None
        """
        warning = False

        # save only package with name and version
        if package.name and package.version:
            if not self.comparator.cmp(package):
                warning = True

            package_data: Dict[str, Any] = {
                "license": package.license,
                "license_version": str(package.license_version),
                "classifier": package.classifier,
            }

            if warning:
                package_data["warning"] = True
            else:
                package_data["warning"] = False

            if self.file.get(package.name) is None:
                self.file[package.name] = {package.version: package_data}
            else:
                if self.file[package.name].get(package.version) is None:
                    self.file[package.name][package.version] = package_data
                else:
                    self._check_duplicity(self.file[package.name].get(package.version), package_data)

            _LOGGER.debug("Add package to OutputCreator: %s", package_data)

        else:
            _LOGGER.debug("The file %s has no package name or version. SKIPPED to create OUTPUT", package.file_path)

    @staticmethod
    def _check_duplicity(old: Dict[str, Any], new: Dict[str, Any]) -> None:
        """
        Check version duplicity, if they don't match create warning in dict.

        :param old: package in class dictionary
        :param new: new package witch want to be added
        :return: None
        """
        if old == new or old["warning"]:
            return
        else:
            for index in old:
                if old[index] is None and old[index] != new[index]:
                    if index != "license_version":
                        old[index] = new[index]

                    if index == "license":
                        old["license_version"] = new["license_version"]

                    _LOGGER.debug("Update package information %s to %s", index, new[index])

                elif old[index] != new[index]:
                    _LOGGER.debug("Found not same duplicity set warning=True")
                    old["warning"] = True

    def is_empty(self) -> bool:
        """Check if variable file is empty."""
        return True if not self.file else False

    def print(self, indent: int = -1) -> None:
        """Print dictionary on STDOUT."""
        _LOGGER.debug("Print on STDOUT final json")
        if indent < 0:
            print(json.dumps(self.file), file=sys.stdout)
        else:
            print(json.dumps(self.file, indent=indent), file=sys.stdout)



#!/usr/bin/env python3
# license-solver
# Copyright(C) 2021 Red Hat, Inc.
#
# This program is free software: you can redistribute it and / or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

"""Tests related to class Package."""

from thoth.license_solver.package import Package


class TestPackage:
    """Test Package."""

    package: Package = Package()

    def test_set_package_name(self) -> None:
        """Test set_package_name function."""
        self.package.set_package_name(None)
        assert self.package.name == ""

        self.package.set_package_name("name")
        assert self.package.name == "name"

        self.package.set_package_name("name_2")
        assert self.package.name == "name_2"

    def test_set_version(self) -> None:
        """Test set_version function."""
        self.package.set_version(None)
        assert self.package.version == ""

        self.package.set_version("1.0")
        assert self.package.version == "1.0"

        self.package.set_version("1.1")
        assert self.package.version == "1.1"

    def test_set_license(self) -> None:
        """Test set_license function."""
        # license found in license_without_version dictionary
        license_without_version = (["MIT License", "MIT", "MIT"], True)
        self.package.set_license(license_without_version)
        assert (
            self.package.license == {"full_name": "MIT License", "identifier_spdx": "MIT", "identifier": "MIT"}
            and self.package.license_version == "LICENSE-WITHOUT-VERSION"
        )

        # license extract version
        license_with_version = (["Apache License 1.1", "Apache-1.1", "Apache 1.1"], True)
        self.package.set_license(license_with_version)
        assert (
            self.package.license
            == {"full_name": "Apache License 1.1", "identifier_spdx": "Apache-1.1", "identifier": "Apache 1.1"}
            and self.package.license_version == "1.1"
        )

        # license without version
        license_with_version = (["Apache License", "Apache", "Apache"], False)
        self.package.set_license(license_with_version)
        assert (
            self.package.license == {"full_name": "Apache License", "identifier_spdx": "Apache", "identifier": "Apache"}
            and self.package.license_version == "UNDETECTED"
        )

        # test license without version
        license_without_version = (list(["LGPL"]), True)
        self.package.set_license(license_without_version)
        assert (
            self.package.license == {"full_name": "LGPL", "identifier_spdx": "UNDETECTED", "identifier": "UNDETECTED"}
            and self.package.license_version == "UNDETECTED"
        )

        # undetected license
        self.package.set_license((list(), False))
        assert (
            self.package.license
            == {"full_name": "UNDETECTED", "identifier": "UNDETECTED", "identifier_spdx": "UNDETECTED"}
            and self.package.license_version == "UNDETECTED"
        )
        self.package.set_license((list(["AAA", "AAA", "AAA"]), True))
        assert self.package.license_version == "UNDETECTED"

    def test_set_license_version(self) -> None:
        """Test set_license_version function."""
        self.package.set_license_version("1")
        assert self.package.license_version == "1"

        self.package.set_license_version("2")
        assert self.package.license_version == "2"

    def test_set_classifier(self) -> None:
        """Test set_classifier function."""
        # test None
        self.package.set_classifier(None)
        assert self.package.classifier == list([["UNDETECTED"]])
        # insert classifier
        self.package.classifier = list()
        self.package.set_classifier(["License :: OSI Approved :: MIT License", "MIT License"])
        assert self.package.classifier == [["License :: OSI Approved :: MIT License", "MIT License"]]
        # try to insert same classifier
        self.package.set_classifier(["License :: OSI Approved :: MIT License", "MIT License"])  # append same classifier
        assert self.package.classifier == [["License :: OSI Approved :: MIT License", "MIT License"]]
        # insert another classifier
        self.package.set_classifier(["License :: OSI Approved :: Apache Software License", "Apache Software License"])
        assert self.package.classifier == [
            ["License :: OSI Approved :: MIT License", "MIT License"],
            ["License :: OSI Approved :: Apache Software License", "Apache Software License"],
        ]

    def test_set_file_path(self) -> None:
        """Test set_file_path function."""
        self.package.set_file_path("path_1")
        assert self.package.file_path == "path_1"

        self.package.set_file_path("path_2")
        assert self.package.file_path == "path_2"



#!/usr/bin/env python3
# license-solver
# Copyright(C) 2021 Red Hat, Inc.
#
# This program is free software: you can redistribute it and / or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

"""Class download classifiers and extract data from them."""

import os
import re
import attr
import logging
from typing import List, Any
from .exceptions import UnableOpenFileData

_LOGGER = logging.getLogger(__name__)


@attr.s(slots=True)
class Classifiers:
    """Class detect all classifiers from downloaded data."""

    data = attr.ib(init=False, type=str)
    classifiers: List[str] = attr.ib(init=True, default=list())
    classifiers_list: List[Any] = list()

    def __attrs_post_init__(self) -> None:
        """INIT method."""
        self.load_data()

    def load_data(self, file_path: str = "data/pypi_classifiers.txt") -> None:
        """Load data for classes variables."""
        file_path = os.path.join(os.path.dirname(os.path.realpath(__file__)), file_path)
        try:
            with open(file_path) as file:
                data = file.read()
                self.data = data
                _LOGGER.debug("File pypi_classifiers.txt was successful loaded")
        except Exception:
            _LOGGER.critical("Could not open/read file: %s", file_path)
            raise UnableOpenFileData

        self._extract_classifiers()

    def _convert_to_list(self) -> None:
        """Covert downloaded string to list."""
        self.classifiers = list(self.data.split("\n"))

    def _extract(self) -> None:
        """Extract licence from classifiers list."""
        for classifier_full in self.classifiers:
            if len(classifier_full) >= 7 and classifier_full.startswith("License"):
                # append licenses to list
                classifier_name = self._extract_name(classifier_full)
                classifier_abbreviation = self._extract_abbreviation(classifier_full)

                li = list()
                li.append(classifier_full)  # full classifier name
                li.append(classifier_name)  # only name without "License :: ..."

                # abbreviation
                if len(classifier_abbreviation) > 0:
                    for abbre in classifier_abbreviation:
                        classifier_no_abbreviation = re.sub(r"\s*\([^()]*\)\s*", "", classifier_name)
                        if classifier_name != classifier_no_abbreviation:
                            li.append(classifier_no_abbreviation)  # name without abbreviation

                        li.append(abbre)  # abbreviation
                        _x = abbre.replace("-", " ")
                        if _x != abbre:
                            li.append(_x)

                self.classifiers_list.append(li)

    def _extract_classifiers(self) -> None:
        """Extract classifiers from downloaded data."""
        self._convert_to_list()
        self._extract()

    @staticmethod
    def _extract_name(classifier: str) -> str:
        """Get licence name from classifier string."""
        data = classifier.split("::")

        if len(data) > 0 and len(data[len(data) - 1]) > 1:
            return data[len(data) - 1].strip()[0:]
        else:
            return ""

    @staticmethod
    def _extract_abbreviation(classifier: str) -> List[str]:
        """Abbreviation for license name."""
        abbreviation = list()
        start = classifier.find("(")
        end = classifier.find(")")
        if start != -1 and end != -1:
            txt = classifier[classifier.find("(") + 1 : classifier.find(")")]
            abbreviation.append(txt)
        return abbreviation



#!/usr/bin/env python3
# license-solver
# Copyright(C) 2021 Red Hat, Inc.
#
# This program is free software: you can redistribute it and / or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

"""Tests related to class Licenses."""
import pytest

from thoth.license_solver.exceptions import UnableOpenFileData
from thoth.license_solver.licenses import Licenses


class TestLicense:
    """Test license."""

    licenses: Licenses = Licenses()

    def test_wrong_file_path(self) -> None:
        """Test loading file and test with wrong file path."""
        with pytest.raises(UnableOpenFileData):
            self.licenses.load_data(file_path="wrong_path")

    def test_init_variables(self) -> None:
        """Test if variables are initialized."""
        assert self.licenses.json_data, "Failed to load JSON data"
        assert self.licenses.licenses
        assert self.licenses.licenses_list

    def test_extract(self):
        """Test extract."""
        self.licenses.json_data = {"licenses": {"aa": "a"}}

        self.licenses._extract()



#!/usr/bin/env python3
# license-solver
# Copyright(C) 2021 Red Hat, Inc.
#
# This program is free software: you can redistribute it and / or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

"""Tests related to class Licenses."""

import pytest

from thoth.license_solver.classifiers import Classifiers
from thoth.license_solver.exceptions import UnableOpenFileData


class TestClassifier:
    """Test classifier."""

    classifier: Classifiers = Classifiers()

    def test_wrong_file_path(self) -> None:
        """Test loading file and test with wrong file path."""
        with pytest.raises(UnableOpenFileData):
            self.classifier.load_data(file_path="wrong_path")

    def test_init_variables(self) -> None:
        """Test if variables are initialized."""
        assert self.classifier.classifiers, "Failed to load JSON data"
        assert self.classifier.classifiers_list

    def test_extract_name(self) -> None:
        """Test extracting classifier name."""
        classifier_txt = "License :: OSI Approved :: MIT License"
        assert self.classifier._extract_name(classifier_txt) == "MIT License", "extract_name does not work properly"
        assert self.classifier._extract_name("") == ""

    def test_extract_abbreviation(self):
        """Text extracting abbreviation."""
        classifier_txt_no_abbreviation = "License :: OSI Approved :: MIT License"
        classifier_txt_with_abbreviation = "License :: OSI Approved :: Mozilla Public License 1.0 (MPL)"

        assert (
            self.classifier._extract_abbreviation(classifier_txt_no_abbreviation) == list()
        ), "Example does not have abbreviation"

        assert self.classifier._extract_abbreviation(classifier_txt_with_abbreviation) == (
            ["MPL"]
        ), "Example does have abbreviation, but no found"



#!/usr/bin/env python3
# license-solver
# Copyright(C) 2021 Red Hat, Inc.
#
# This program is free software: you can redistribute it and / or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

"""Tests related to class Solver."""

import os
import pytest
from thoth.license_solver.solver import Solver


class TestSolver:
    """Test solver."""

    solver: Solver = Solver()

    # test get_license_group function
    def test_get_license_group_unknown(self) -> None:
        """Test detecting license group for unknown."""
        assert self.solver._get_license_group(None) == (list(["UNDETECTED"]), False)
        assert self.solver._get_license_group("UNKNOWN") == (list(["UNKNOWN"]), False)
        assert self.solver._get_license_group("uNkNOWN") == (list(["UNKNOWN"]), False)

    def test_get_license_group_found_in_license_list(self) -> None:
        """Test detecting license group in license."""
        bsd = ['BSD 4-Clause "Original" or "Old" License', "BSD-4-Clause", "BSD 4 Clause"]
        assert self.solver._get_license_group(bsd[0]) == (bsd, True)
        assert self.solver._get_license_group(bsd[1]) == (bsd, True)
        assert self.solver._get_license_group(bsd[2]) == (bsd, True)

        bsd_with_random_lowercase = ['bsd 4-ClaUse "Original" or "Old" License', "BsD-4-ClaUse", "BsD 4 Clause"]
        assert self.solver._get_license_group(bsd_with_random_lowercase[0]) == (bsd, True)
        assert self.solver._get_license_group(bsd_with_random_lowercase[1]) == (bsd, True)
        assert self.solver._get_license_group(bsd_with_random_lowercase[2]) == (bsd, True)

        # find license without version
        assert self.solver._get_license_group("Apache") == (list(["Apache"]), True)

    def test_get_license_group_in_file(self) -> None:
        """Test detecting license group in dictionary."""
        apache = ["Apache License 1.0", "Apache-1.0", "Apache 1.0"]
        assert self.solver._get_license_group("apache-1") == (apache, True)
        assert self.solver._get_license_group("APACHE-1") == (apache, True)

    def test_get_license_group_not_fount(self) -> None:
        """Test detecting license group not found."""
        assert self.solver._get_license_group("89fdslkj94") == (list(["UNDETECTED"]), False)

    # test get_classifier_group function
    def test_get_classifier_group_none(self) -> None:
        """Test detecting classifier group which output None."""
        assert self.solver._get_classifier_group(None) is None

    def test_get_classifier_group_found_in_classifier_list(self) -> None:
        """Test detecting classifier group found in classifier list."""
        classifier_input = ["License :: Aladdin Free Public License (AFPL)"]
        classifier_output = [
            "License :: Aladdin Free Public License (AFPL)",
            "Aladdin Free Public License (AFPL)",
            "Aladdin Free Public License",
            "AFPL",
        ]
        assert self.solver._get_classifier_group(classifier_input) == classifier_output

        classifier_input_2 = ["AFPL"]
        assert self.solver._get_classifier_group(classifier_input_2) == classifier_output

        # tests with lowercase
        classifier_input_3 = ["aFpl"]
        assert self.solver._get_classifier_group(classifier_input_3) == classifier_output

    @pytest.fixture(autouse=True)
    def capsys(self, capsys):
        """Capsys for Pytest."""
        self.capsys = capsys

    def test_solver(self):
        """Test Solver."""
        file_path = os.path.join(
            os.path.dirname(os.path.realpath(__file__)), "test_files", "solver", "test_solver_files"
        )
        self.solver.solve_from_directory(file_path)
        self.solver.print_output()

        out, err = self.capsys.readouterr()

        assert str(out) == str(
            '{"moto": {"1.3.15.dev221": {"license": {"full_name": "Apache", '
            '"identifier_spdx": "UNDETECTED", "identifier": "UNDETECTED"}, '
            '"license_version": "UNDETECTED", "classifier": [["License :: OSI Approved :: '
            'Apache Software License", "Apache Software License"]], "warning": false}}, '
            '"itk-segmentation": {"4.13.2": {"license": {"full_name": "Apache", '
            '"identifier_spdx": "UNDETECTED", "identifier": "UNDETECTED"}, '
            '"license_version": "UNDETECTED", "classifier": [["License :: OSI Approved :: '
            'Apache Software License", "Apache Software License"]], "warning": false}}, '
            '"great-expectations": {"0.11.0b0": {"license": {"full_name": "Apache License '
            '2.0", "identifier_spdx": "Apache-2.0", "identifier": "Apache 2.0"}, '
            '"license_version": "2.0", "classifier": [["License :: OSI Approved :: Apache '
            'Software License", "Apache Software License"]], "warning": false}}, '
            '"SQLAlchemy": {"1.3.15": {"license": {"full_name": "Apache License 1.1", '
            '"identifier_spdx": "Apache-1.1", "identifier": "Apache 1.1"}, '
            '"license_version": "1.1", "classifier": [["License :: OSI Approved :: MIT '
            'License", "MIT License"]], "warning": true}, "9.10": {"license": '
            '{"full_name": "UNDETECTED", "identifier_spdx": "UNDETECTED", "identifier": '
            '"UNDETECTED"}, "license_version": "UNDETECTED", "classifier": [["License :: '
            'OSI Approved :: MIT License", "MIT License"]], "warning": true}, "9.9": '
            '{"license": {"full_name": "Apache License 2.0", "identifier_spdx": '
            '"Apache-2.0", "identifier": "Apache 2.0"}, "license_version": "2.0", '
            '"classifier": [["License :: OSI Approved :: MIT License", "MIT License"]], '
            '"warning": true}}, "polyaxon-client": {"0.6.0": {"license": {"full_name": '
            '"MIT License", "identifier_spdx": "MIT", "identifier": "MIT"}, '
            '"license_version": "LICENSE-WITHOUT-VERSION", "classifier": '
            '[["UNDETECTED"]], "warning": false}}, "setuptools-scm": {"1.15.4": '
            '{"license": {"full_name": "MIT License", "identifier_spdx": "MIT", '
            '"identifier": "MIT"}, "license_version": "LICENSE-WITHOUT-VERSION", '
            '"classifier": [["License :: OSI Approved :: MIT License", "MIT License"]], '
            '"warning": false}}, "path.py": {"2.6.1": {"license": {"full_name": "MIT '
            'License", "identifier_spdx": "MIT", "identifier": "MIT"}, "license_version": '
            '"LICENSE-WITHOUT-VERSION", "classifier": [["License :: OSI Approved :: MIT '
            'License", "MIT License"]], "warning": false}}, "cwltool": '
            '{"1.0.20160108200940": {"license": {"full_name": "Apache License 2.0", '
            '"identifier_spdx": "Apache-2.0", "identifier": "Apache 2.0"}, '
            '"license_version": "2.0", "classifier": [["UNDETECTED"]], "warning": '
            'false}}, "django-redis-cache": {"1.5.1": {"license": {"full_name": '
            '"UNKNOWN", "identifier_spdx": "UNDETECTED", "identifier": "UNDETECTED"}, '
            '"license_version": "UNDETECTED", "classifier": [["UNDETECTED"]], "warning": '
            'false}}, "oslotest": {"4.2.0": {"license": {"full_name": "UNKNOWN", '
            '"identifier_spdx": "UNDETECTED", "identifier": "UNDETECTED"}, '
            '"license_version": "UNDETECTED", "classifier": [["License :: OSI Approved :: '
            'Apache Software License", "Apache Software License"]], "warning": false}}, '
            '"django-webpack-loader": {"0.0.6.1": {"license": {"full_name": "UNKNOWN", '
            '"identifier_spdx": "UNDETECTED", "identifier": "UNDETECTED"}, '
            '"license_version": "UNDETECTED", "classifier": [["License :: OSI Approved :: '
            'MIT License", "MIT License"]], "warning": false}}, "trimesh": {"2.29.9": '
            '{"license": {"full_name": "MIT License", "identifier_spdx": "MIT", '
            '"identifier": "MIT"}, "license_version": "LICENSE-WITHOUT-VERSION", '
            '"classifier": [["License :: OSI Approved :: MIT License", "MIT License"]], '
            '"warning": false}}}\n'
        )
